'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _Network = require('cashlib/lib/Network');

var _Network2 = _interopRequireDefault(_Network);

var _state = require('./state');

var _state2 = _interopRequireDefault(_state);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const MAX_HISTORY = 25;

class Connection {

  constructor(network) {
    this._isConnected = false;
    this._network = network;
    this._host = '127.0.0.1';
    this._auth = '';
    const datadir = Connection.getDefaultDataDir();
    switch (network.label) {
      case 'mainnet':
        this._port = 8332;
        this._cookieFile = _path2.default.join(datadir, '.cookie');
        break;
      case 'testnet':
        this._port = 18332;
        this._cookieFile = _path2.default.join(datadir, 'testnet3', '.cookie');
        break;
      case 'regtest':
        this._port = 18332;
        this._cookieFile = _path2.default.join(datadir, 'regtest', '.cookie');
        break;
      case 'nol':
        this._port = 9332;
        this._cookieFile = _path2.default.join(datadir, 'nol', '.cookie');
        break;
      default:
        throw new Error(`Unrecognized network ${network.label}`);
    }
  }

  // Readonly members
  get network() {
    return this._network;
  }
  get isConnected() {
    return this._isConnected;
  }

  // Settable members
  get host() {
    return this._host;
  }
  get port() {
    return this._port;
  }
  get cookieFile() {
    return this._cookieFile;
  }
  get user() {
    return this._user;
  }
  get password() {
    return this._password;
  }

  _clearConnection() {
    this._auth = '';
    this._isConnected = false;
  }

  set host(host) {
    this._clearConnection();
    this._host = host;
  }

  set port(port) {
    this._clearConnection();
    this._port = port;
  }

  set cookieFile(cookieFile) {
    this._clearConnection();
    this._cookieFile = cookieFile;
  }

  set password(password) {
    this._clearConnection();
    this._password = password;
  }

  set user(user) {
    this._clearConnection();
    this._user = user;
  }

  get auth() {
    if (!this._auth) {
      if (this._cookieFile) {
        try {
          this._auth = _fs2.default.readFileSync(this._cookieFile, { encoding: 'utf8' });
        } catch (err) {
          throw new Error(`Error reading file '${this._cookieFile}'`);
        }
      } else {
        this._auth = `${this._user}:${this._password}`;
      }
    }
    return this._auth;
  }

  async connect() {
    await _state2.default.rpc.request('getinfo');
    this._isConnected = true;
    Connection.addHistory(this);
  }

  static getDefaultDataDir() {
    let datadir;
    switch (process.platform) {
      case 'win32':
        // Windows >= Vista:
        if (process.env.user) {
          datadir = `C:\\Users\\${process.env.user}\\AppData\\Roaming\\Bitcoin`;
        } else {
          throw new Error('Unable to detect data directory for current user');
        }
        break;
      case 'linux':
        // Unix:
        datadir = '~/.bitcoin';
        break;
      case 'darwin':
        // Mac:
        datadir = '~/Library/Application Support/Bitcoin';
        break;
      default:
        throw new Error(`Unrecognized platform '${process.platform}'`);
    }
    return datadir;
  }

  static getHistory() {
    let history = [];
    const stored = _state2.default.localStorage.getItem('connectionHistory');
    if (stored) {
      history = JSON.parse(stored);
    }
    return history;
  }

  static addHistory(connection) {
    const newRec = {
      network: connection.network.label,
      host: connection.host,
      port: connection.port,
      cookieFile: connection.cookieFile,
      user: connection.user,
      password: connection.password
    };

    let history = this.getHistory();

    // Remove any earlier connections with identical settings
    history = history.filter(old => old.network !== connection.network.label || old.host !== connection.host || old.port !== connection.port || old.cookieFile !== connection.cookieFile || old.user !== connection.user);

    // Add connection to top of history
    if (history.length) {
      history.unshift(newRec);
    } else {
      history.push(newRec);
    }

    if (history.length > MAX_HISTORY) {
      history.splice(MAX_HISTORY);
    }

    _state2.default.localStorage.setItem('connectionHistory', JSON.stringify(history));
  }
}
exports.default = Connection;